<?php

namespace Drupal\neutral_nodes_disable_routing\Routing;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Routing\RequestContext;
use Symfony\Cmf\Component\Routing\NestedMatcher\RouteFilterInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Filters variant routes.
 *
 * Each variant for a single page has a unique route for the same path, and
 * needs to be filtered. Here is where we run variant selection, which requires
 * gathering contexts.
 */
class RouteFilter implements RouteFilterInterface {

  /**
   * The list of available route filters.
   *
   * @var \Symfony\Cmf\Component\Routing\NestedMatcher\RouteFilterInterface[]
   */
  protected $filters = [];

  /**
   * The page variant storage.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entity;

  /**
   * The current path stack.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * The current request context.
   *
   * @var \Drupal\Core\Routing\RequestContext
   */
  protected $request;

  /**
   * Constructs a new VariantRouteFilter.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity
   * @param \Drupal\Core\Path\CurrentPathStack $current_path
   *   The current path stack.
   * @param \Drupal\Core\Routing\RequestContext $request
   *   The current request stack.
   * @internal param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager The entity type manager.*   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity, CurrentPathStack $current_path, RequestContext $request) {
    $this->entity = $entity;
    $this->currentPath = $current_path;
    $this->request = $request;
  }

  /**
   * {@inheritdoc}
   *
   * Ensures only one page manager route remains in the collection.
   */
  public function filter(RouteCollection $collection, Request $request) {
    $routes = $collection->all();
    // Create a new route collection by iterating over the sorted routes, using
    // the overridden_route_name if available.
    $result_collection = new RouteCollection();


    array_filter($routes, function (Route $route) use (&$result_collection, $request) {
      if (!$route->hasDefault('neutral_nodes_disable_routing.node_view')) {
        return FALSE;
      }
      // We'll retrieve the current node bundle.
      $path = $this->currentPath->getPath($request);
      $bundle = '';
      // See \Symfony\Component\Routing\Matcher\UrlMatcher::matchCollection().
      preg_match($route->compile()->getRegex(), $path, $matches);
      if (isset($matches['node'])) {
        $node_storage = $this->entity->getStorage('node');
        $node = $node_storage->load($matches['node']);
        if ($node) {
          $bundle = $node->bundle();
        }
        if ($bundle && $route->getDefault('neutral_nodes_disable_routing.node_view') == $bundle) {
          $result_collection->add("neutral_nodes_disable_routing.node_view.$bundle", $route);
        }
      }
      return ($route->getDefault('neutral_nodes_disable_routing.node_view') == $bundle);
    });

    return ($result_collection->count() > 0) ? $result_collection : $collection;
  }

  /**
   * {@inheritdoc}
   */
  public function applies(Route $route) {
    return !empty($route->getMethods());
  }

  /**
   * Adds a route filter to the list of used route filters.
   *
   * @param \Symfony\Cmf\Component\Routing\NestedMatcher\RouteFilterInterface $route_filter
   *   A route filter.
   * @param int $priority
   *   (optional) The priority of the filter. Higher number filters will be used
   *   first.
   *
   * @return $this
   */
  public function addRouteFilter(RouteFilterInterface $route_filter, $priority = 0) {
    $this->filters[$priority][] = $route_filter;

    return $this;
  }
}
